@@ -22,9 +22,7 @@
 'use strict';
 
 const {
-  ArrayPrototypePush,
   JSONParse,
-  RegExpPrototypeSymbolReplace,
 } = primordials;
 
 const tls = require('tls');
@@ -133,21 +131,21 @@ function translatePeerCertificate(c) {
     c.infoAccess = { __proto__: null };
 
     // XXX: More key validation?
-    RegExpPrototypeSymbolReplace(/([^\n:]*):([^\n]*)(?:\n|$)/g, info,
-                                 (all, key, val) => {
-                                   if (val.charCodeAt(0) === 0x22) {
-                                     // The translatePeerCertificate function is only
-                                     // used on internally created legacy certificate
-                                     // objects, and any value that contains a quote
-                                     // will always be a valid JSON string literal,
-                                     // so this should never throw.
-                                     val = JSONParse(val);
-                                   }
-                                   if (key in c.infoAccess)
-                                     ArrayPrototypePush(c.infoAccess[key], val);
-                                   else
-                                     c.infoAccess[key] = [val];
-                                 });
+    info.replace(/([^\n:]*):([^\n]*)(?:\n|$)/g,
+                 (all, key, val) => {
+                   if (val.charCodeAt(0) === 0x22) {
+                     // The translatePeerCertificate function is only
+                     // used on internally created legacy certificate
+                     // objects, and any value that contains a quote
+                     // will always be a valid JSON string literal,
+                     // so this should never throw.
+                     val = JSONParse(val);
+                   }
+                   if (key in c.infoAccess)
+                     c.infoAccess[key].push(val);
+                   else
+                     c.infoAccess[key] = [val];
+                 });
   }
   return c;
 }
