@@ -2,8 +2,6 @@
 
 const {
   ObjectPrototypeHasOwnProperty,
-  PromisePrototypeThen,
-  PromiseResolve,
   RegExpPrototypeExec,
   SafeSet,
   StringPrototypeCharCodeAt,
@@ -17,8 +15,7 @@ const {
   mimeToFormat,
 } = require('internal/modules/esm/formats');
 
-const experimentalNetworkImports =
-  getOptionValue('--experimental-network-imports');
+const detectModule = getOptionValue('--experimental-detect-module');
 const { containsModuleSyntax } = internalBinding('contextify');
 const { getPackageScopeConfig, getPackageType } = require('internal/modules/package_json_reader');
 const { fileURLToPath } = require('internal/url');
@@ -28,11 +25,20 @@ const protocolHandlers = {
   '__proto__': null,
   'data:': getDataProtocolModuleFormat,
   'file:': getFileProtocolModuleFormat,
-  'http:': getHttpProtocolModuleFormat,
-  'https:': getHttpProtocolModuleFormat,
   'node:'() { return 'builtin'; },
 };
 
+/**
+ * Determine whether the given ambiguous source contains CommonJS or ES module syntax.
+ * @param {string | Buffer | undefined} source
+ * @param {URL} url
+ */
+function detectModuleFormat(source, url) {
+  if (!source) { return detectModule ? null : 'commonjs'; }
+  if (!detectModule) { return 'commonjs'; }
+  return containsModuleSyntax(`${source}`, fileURLToPath(url), url) ? 'module' : 'commonjs';
+}
+
 /**
  * @param {URL} parsed
  * @returns {string | null}
@@ -83,6 +89,18 @@ function underNodeModules(url) {
 }
 
 let typelessPackageJsonFilesWarnedAbout;
+function warnTypelessPackageJsonFile(pjsonPath, url) {
+  typelessPackageJsonFilesWarnedAbout ??= new SafeSet();
+  if (!typelessPackageJsonFilesWarnedAbout.has(pjsonPath)) {
+    const warning = `${url} parsed as an ES module because module syntax was detected;` +
+      ` to avoid the performance penalty of syntax detection, add "type": "module" to ${pjsonPath}`;
+    process.emitWarning(warning, {
+      code: 'MODULE_TYPELESS_PACKAGE_JSON',
+    });
+    typelessPackageJsonFilesWarnedAbout.add(pjsonPath);
+  }
+}
+
 /**
  * @param {URL} url
  * @param {{parentURL: string; source?: Buffer}} context
@@ -112,26 +130,50 @@ function getFileProtocolModuleFormat(url, context = { __proto__: null }, ignoreE
       default: { // The user did not pass `--experimental-default-type`.
         // `source` is undefined when this is called from `defaultResolve`;
         // but this gets called again from `defaultLoad`/`defaultLoadSync`.
-        if (getOptionValue('--experimental-detect-module')) {
-          const format = source ?
-            (containsModuleSyntax(`${source}`, fileURLToPath(url), url) ? 'module' : 'commonjs') :
-            null;
-          if (format === 'module') {
-            // This module has a .js extension, a package.json with no `type` field, and ESM syntax.
-            // Warn about the missing `type` field so that the user can avoid the performance penalty of detection.
-            typelessPackageJsonFilesWarnedAbout ??= new SafeSet();
-            if (!typelessPackageJsonFilesWarnedAbout.has(pjsonPath)) {
-              const warning = `${url} parsed as an ES module because module syntax was detected;` +
-              ` to avoid the performance penalty of syntax detection, add "type": "module" to ${pjsonPath}`;
-              process.emitWarning(warning, {
-                code: 'MODULE_TYPELESS_PACKAGE_JSON',
-              });
-              typelessPackageJsonFilesWarnedAbout.add(pjsonPath);
-            }
-          }
-          return format;
+        // For ambiguous files (no type field, .js extension) we return
+        // undefined from `resolve` and re-run the check in `load`.
+        const format = detectModuleFormat(source, url);
+        if (format === 'module') {
+          // This module has a .js extension, a package.json with no `type` field, and ESM syntax.
+          // Warn about the missing `type` field so that the user can avoid the performance penalty of detection.
+          warnTypelessPackageJsonFile(pjsonPath, url);
         }
-        return 'commonjs';
+        return format;
+      }
+    }
+  }
+  if (ext === '.ts' && getOptionValue('--experimental-strip-types')) {
+    const { type: packageType, pjsonPath } = getPackageScopeConfig(url);
+    if (packageType !== 'none') {
+      return `${packageType}-typescript`;
+    }
+    // The controlling `package.json` file has no `type` field.
+    switch (getOptionValue('--experimental-default-type')) {
+      case 'module': { // The user explicitly passed `--experimental-default-type=module`.
+        // An exception to the type flag making ESM the default everywhere is that package scopes under `node_modules`
+        // should retain the assumption that a lack of a `type` field means CommonJS.
+        return underNodeModules(url) ? 'commonjs-typescript' : 'module-typescript';
+      }
+      case 'commonjs': { // The user explicitly passed `--experimental-default-type=commonjs`.
+        return 'commonjs-typescript';
+      }
+      default: { // The user did not pass `--experimental-default-type`.
+        // `source` is undefined when this is called from `defaultResolve`;
+        // but this gets called again from `defaultLoad`/`defaultLoadSync`.
+        let parsedSource;
+        if (source) {
+          const { stripTypeScriptTypes } = require('internal/modules/helpers');
+          parsedSource = stripTypeScriptTypes(source, url);
+        }
+        const detectedFormat = detectModuleFormat(parsedSource, url);
+        // When source is undefined, default to module-typescript.
+        const format = detectedFormat ? `${detectedFormat}-typescript` : 'module-typescript';
+        if (format === 'module-typescript') {
+          // This module has a .js extension, a package.json with no `type` field, and ESM syntax.
+          // Warn about the missing `type` field so that the user can avoid the performance penalty of detection.
+          warnTypelessPackageJsonFile(pjsonPath, url);
+        }
+        return format;
       }
     }
   }
@@ -154,15 +196,14 @@ function getFileProtocolModuleFormat(url, context = { __proto__: null }, ignoreE
         return 'commonjs';
       }
       default: { // The user did not pass `--experimental-default-type`.
-        if (getOptionValue('--experimental-detect-module')) {
-          if (!source) { return null; }
-          const format = getFormatOfExtensionlessFile(url);
-          if (format === 'module') {
-            return containsModuleSyntax(`${source}`, fileURLToPath(url), url) ? 'module' : 'commonjs';
-          }
+        if (!source) {
+          return null;
+        }
+        const format = getFormatOfExtensionlessFile(url);
+        if (format === 'wasm') {
           return format;
         }
-        return 'commonjs';
+        return detectModuleFormat(source, url);
       }
     }
   }
@@ -176,23 +217,6 @@ function getFileProtocolModuleFormat(url, context = { __proto__: null }, ignoreE
   throw new ERR_UNKNOWN_FILE_EXTENSION(ext, filepath);
 }
 
-/**
- * @param {URL} url
- * @param {{parentURL: string}} context
- * @returns {Promise<string> | undefined} only works when enabled
- */
-function getHttpProtocolModuleFormat(url, context) {
-  if (experimentalNetworkImports) {
-    const { fetchModule } = require('internal/modules/esm/fetch_module');
-    return PromisePrototypeThen(
-      PromiseResolve(fetchModule(url, context)),
-      (entry) => {
-        return mimeToFormat(entry.headers['content-type']);
-      },
-    );
-  }
-}
-
 /**
  * @param {URL} url
  * @param {{parentURL: string}} context
