@@ -33,9 +33,9 @@ const {
   ERR_WORKER_UNSERIALIZABLE_ERROR,
 } = require('internal/errors').codes;
 const { exitCodes: { kUnsettledTopLevelAwait } } = internalBinding('errors');
-const { URL } = require('internal/url');
+const { URLParse } = require('internal/url');
 const { canParse: URLCanParse } = internalBinding('url');
-const { receiveMessageOnPort, isMainThread } = require('worker_threads');
+const { receiveMessageOnPort } = require('worker_threads');
 const {
   isAnyArrayBuffer,
   isArrayBufferView,
@@ -145,13 +145,15 @@ class Hooks {
    * @param {any} [data] Arbitrary data to be passed from the custom
    * loader (user-land) to the worker.
    */
-  async register(urlOrSpecifier, parentURL, data) {
+  async register(urlOrSpecifier, parentURL, data, isInternal) {
     const cascadedLoader = require('internal/modules/esm/loader').getOrInitializeCascadedLoader();
-    const keyedExports = await cascadedLoader.import(
-      urlOrSpecifier,
-      parentURL,
-      kEmptyObject,
-    );
+    const keyedExports = isInternal ?
+      require(urlOrSpecifier) :
+      await cascadedLoader.import(
+        urlOrSpecifier,
+        parentURL,
+        kEmptyObject,
+      );
     await this.addCustomLoader(urlOrSpecifier, keyedExports, data);
   }
 
@@ -403,11 +405,7 @@ class Hooks {
 
     let responseURLObj;
     if (typeof responseURL === 'string') {
-      try {
-        responseURLObj = new URL(responseURL);
-      } catch {
-        // responseURLObj not defined will throw in next branch.
-      }
+      responseURLObj = URLParse(responseURL);
     }
 
     if (responseURLObj?.href !== responseURL) {
@@ -482,8 +480,6 @@ class HooksProxy {
    */
   #worker;
 
-  #portToHooksThread;
-
   /**
    * The last notification ID received from the worker. This is used to detect
    * if the worker has already sent a notification before putting the main
@@ -501,38 +497,26 @@ class HooksProxy {
   #isReady = false;
 
   constructor() {
-    const { InternalWorker, hooksPort } = require('internal/worker');
+    const { InternalWorker } = require('internal/worker');
+    MessageChannel ??= require('internal/worker/io').MessageChannel;
+
     const lock = new SharedArrayBuffer(SHARED_MEMORY_BYTE_LENGTH);
     this.#lock = new Int32Array(lock);
 
-    if (isMainThread) {
-      // Main thread is the only one that creates the internal single hooks worker
-      this.#worker = new InternalWorker(loaderWorkerId, {
-        stderr: false,
-        stdin: false,
-        stdout: false,
-        trackUnmanagedFds: false,
-        workerData: {
-          lock,
-        },
-      });
-      this.#worker.unref(); // ! Allows the process to eventually exit.
-      this.#worker.on('exit', process.exit);
-      this.#portToHooksThread = this.#worker;
-    } else {
-      this.#portToHooksThread = hooksPort;
-    }
+    this.#worker = new InternalWorker(loaderWorkerId, {
+      stderr: false,
+      stdin: false,
+      stdout: false,
+      trackUnmanagedFds: false,
+      workerData: {
+        lock,
+      },
+    });
+    this.#worker.unref(); // ! Allows the process to eventually exit.
+    this.#worker.on('exit', process.exit);
   }
 
   waitForWorker() {
-    // There is one Hooks instance for each worker thread. But only one of these Hooks instances
-    // has an InternalWorker. That was the Hooks instance created for the main thread.
-    // It means for all Hooks instances that are not on the main thread => they are ready because they
-    // delegate to the single InternalWorker anyway.
-    if (!isMainThread) {
-      return;
-    }
-
     if (!this.#isReady) {
       const { kIsOnline } = require('internal/worker');
       if (!this.#worker[kIsOnline]) {
@@ -549,37 +533,6 @@ class HooksProxy {
     }
   }
 
-  #postMessageToWorker(method, type, transferList, args) {
-    this.waitForWorker();
-
-    MessageChannel ??= require('internal/worker/io').MessageChannel;
-
-    const {
-      port1: fromHooksThread,
-      port2: toHooksThread,
-    } = new MessageChannel();
-
-    // Pass work to the worker.
-    debug(`post ${type} message to worker`, { method, args, transferList });
-    const usedTransferList = [toHooksThread];
-    if (transferList) {
-      ArrayPrototypePushApply(usedTransferList, transferList);
-    }
-
-    this.#portToHooksThread.postMessage(
-      {
-        __proto__: null,
-        args,
-        lock: this.#lock,
-        method,
-        port: toHooksThread,
-      },
-      usedTransferList,
-    );
-
-    return fromHooksThread;
-  }
-
   /**
    * Invoke a remote method asynchronously.
    * @param {string} method Method to invoke
@@ -588,7 +541,22 @@ class HooksProxy {
    * @returns {Promise<any>}
    */
   async makeAsyncRequest(method, transferList, ...args) {
-    const fromHooksThread = this.#postMessageToWorker(method, 'Async', transferList, args);
+    this.waitForWorker();
+
+    MessageChannel ??= require('internal/worker/io').MessageChannel;
+    const asyncCommChannel = new MessageChannel();
+
+    // Pass work to the worker.
+    debug('post async message to worker', { method, args, transferList });
+    const finalTransferList = [asyncCommChannel.port2];
+    if (transferList) {
+      ArrayPrototypePushApply(finalTransferList, transferList);
+    }
+    this.#worker.postMessage({
+      __proto__: null,
+      method, args,
+      port: asyncCommChannel.port2,
+    }, finalTransferList);
 
     if (this.#numberOfPendingAsyncResponses++ === 0) {
       // On the next lines, the main thread will await a response from the worker thread that might
@@ -597,11 +565,7 @@ class HooksProxy {
       // However we want to keep the process alive until the worker thread responds (or until the
       // event loop of the worker thread is also empty), so we ref the worker until we get all the
       // responses back.
-      if (this.#worker) {
-        this.#worker.ref();
-      } else {
-        this.#portToHooksThread.ref();
-      }
+      this.#worker.ref();
     }
 
     let response;
@@ -610,26 +574,18 @@ class HooksProxy {
       await AtomicsWaitAsync(this.#lock, WORKER_TO_MAIN_THREAD_NOTIFICATION, this.#workerNotificationLastId).value;
       this.#workerNotificationLastId = AtomicsLoad(this.#lock, WORKER_TO_MAIN_THREAD_NOTIFICATION);
 
-      response = receiveMessageOnPort(fromHooksThread);
+      response = receiveMessageOnPort(asyncCommChannel.port1);
     } while (response == null);
     debug('got async response from worker', { method, args }, this.#lock);
 
     if (--this.#numberOfPendingAsyncResponses === 0) {
       // We got all the responses from the worker, its job is done (until next time).
-      if (this.#worker) {
-        this.#worker.unref();
-      } else {
-        this.#portToHooksThread.unref();
-      }
+      this.#worker.unref();
     }
 
-    if (response.message.status === 'exit') {
-      process.exit(response.message.body);
-    }
-
-    fromHooksThread.close();
-
-    return this.#unwrapMessage(response);
+    const body = this.#unwrapMessage(response);
+    asyncCommChannel.port1.close();
+    return body;
   }
 
   /**
@@ -640,7 +596,11 @@ class HooksProxy {
    * @returns {any}
    */
   makeSyncRequest(method, transferList, ...args) {
-    const fromHooksThread = this.#postMessageToWorker(method, 'Sync', transferList, args);
+    this.waitForWorker();
+
+    // Pass work to the worker.
+    debug('post sync message to worker', { method, args, transferList });
+    this.#worker.postMessage({ __proto__: null, method, args }, transferList);
 
     let response;
     do {
@@ -649,7 +609,7 @@ class HooksProxy {
       AtomicsWait(this.#lock, WORKER_TO_MAIN_THREAD_NOTIFICATION, this.#workerNotificationLastId);
       this.#workerNotificationLastId = AtomicsLoad(this.#lock, WORKER_TO_MAIN_THREAD_NOTIFICATION);
 
-      response = receiveMessageOnPort(fromHooksThread);
+      response = this.#worker.receiveMessageSync();
     } while (response == null);
     debug('got sync response from worker', { method, args });
     if (response.message.status === 'never-settle') {
@@ -657,9 +617,6 @@ class HooksProxy {
     } else if (response.message.status === 'exit') {
       process.exit(response.message.body);
     }
-
-    fromHooksThread.close();
-
     return this.#unwrapMessage(response);
   }
 
