@@ -1,7 +1,6 @@
 'use strict';
 
 const {
-  ArrayPrototypePush,
   RegExpPrototypeExec,
   decodeURIComponent,
 } = primordials;
@@ -12,18 +11,20 @@ const { validateAttributes, emitImportAssertionWarning } = require('internal/mod
 const { getOptionValue } = require('internal/options');
 const { readFileSync } = require('fs');
 
-const experimentalNetworkImports =
-  getOptionValue('--experimental-network-imports');
 const defaultType =
   getOptionValue('--experimental-default-type');
 
 const { Buffer: { from: BufferFrom } } = require('buffer');
+const {
+  isUnderNodeModules,
+} = require('internal/modules/helpers');
 
 const { URL } = require('internal/url');
 const {
   ERR_INVALID_URL,
   ERR_UNKNOWN_MODULE_FORMAT,
   ERR_UNSUPPORTED_ESM_URL_SCHEME,
+  ERR_UNSUPPORTED_NODE_MODULES_TYPE_STRIPPING,
 } = require('internal/errors').codes;
 
 const DATA_URL_PATTERN = /^[^/]+\/[^,;]+(?:[^,]*?)(;base64)?,([\s\S]*)$/;
@@ -35,7 +36,7 @@ const DATA_URL_PATTERN = /^[^/]+\/[^,;]+(?:[^,]*?)(;base64)?,([\s\S]*)$/;
  */
 async function getSource(url, context) {
   const { protocol, href } = url;
-  let responseURL = href;
+  const responseURL = href;
   let source;
   if (protocol === 'file:') {
     const { readFile: readFileAsync } = require('internal/fs/promises').exports;
@@ -47,19 +48,8 @@ async function getSource(url, context) {
     }
     const { 1: base64, 2: body } = match;
     source = BufferFrom(decodeURIComponent(body), base64 ? 'base64' : 'utf8');
-  } else if (experimentalNetworkImports && (
-    protocol === 'https:' ||
-    protocol === 'http:'
-  )) {
-    const { fetchModule } = require('internal/modules/esm/fetch_module');
-    const res = await fetchModule(url, context);
-    source = await res.body;
-    responseURL = res.resolvedHREF;
   } else {
     const supportedSchemes = ['file', 'data'];
-    if (experimentalNetworkImports) {
-      ArrayPrototypePush(supportedSchemes, 'http', 'https');
-    }
     throw new ERR_UNSUPPORTED_ESM_URL_SCHEME(url, supportedSchemes);
   }
   return { __proto__: null, responseURL, source };
@@ -117,7 +107,7 @@ async function defaultLoad(url, context = kEmptyObject) {
 
   const urlInstance = new URL(url);
 
-  throwIfUnsupportedURLScheme(urlInstance, experimentalNetworkImports);
+  throwIfUnsupportedURLScheme(urlInstance);
 
   if (urlInstance.protocol === 'node:') {
     source = null;
@@ -147,6 +137,12 @@ async function defaultLoad(url, context = kEmptyObject) {
     format = 'commonjs-sync';
   }
 
+  if (getOptionValue('--experimental-strip-types') &&
+    (format === 'module-typescript' || format === 'commonjs-typescript') &&
+    isUnderNodeModules(url)) {
+    throw new ERR_UNSUPPORTED_NODE_MODULES_TYPE_STRIPPING(url);
+  }
+
   return {
     __proto__: null,
     format,
@@ -214,9 +210,8 @@ function defaultLoadSync(url, context = kEmptyObject) {
  * throws an error if the protocol is not one of the protocols
  * that can be loaded in the default loader
  * @param {URL} parsed
- * @param {boolean} experimentalNetworkImports
  */
-function throwIfUnsupportedURLScheme(parsed, experimentalNetworkImports) {
+function throwIfUnsupportedURLScheme(parsed) {
   // Avoid accessing the `protocol` property due to the lazy getters.
   const protocol = parsed?.protocol;
   if (
@@ -225,17 +220,11 @@ function throwIfUnsupportedURLScheme(parsed, experimentalNetworkImports) {
     protocol !== 'data:' &&
     protocol !== 'node:' &&
     (
-      !experimentalNetworkImports ||
-      (
-        protocol !== 'https:' &&
-        protocol !== 'http:'
-      )
+      protocol !== 'https:' &&
+      protocol !== 'http:'
     )
   ) {
     const schemes = ['file', 'data', 'node'];
-    if (experimentalNetworkImports) {
-      ArrayPrototypePush(schemes, 'https', 'http');
-    }
     throw new ERR_UNSUPPORTED_ESM_URL_SCHEME(parsed, schemes);
   }
 }
