@@ -30,7 +30,7 @@ const {
 } = require('internal/errors').codes;
 const { getOptionValue } = require('internal/options');
 const { isURL, pathToFileURL, URLParse } = require('internal/url');
-const { emitExperimentalWarning, kEmptyObject } = require('internal/util');
+const { kEmptyObject } = require('internal/util');
 const {
   compileSourceTextModule,
   getDefaultConditions,
@@ -145,9 +145,6 @@ class ModuleLoader {
   #customizations;
 
   constructor(customizations) {
-    if (getOptionValue('--experimental-network-imports')) {
-      emitExperimentalWarning('Network Imports');
-    }
     this.setCustomizations(customizations);
   }
 
@@ -494,7 +491,7 @@ class ModuleLoader {
   /**
    * @see {@link CustomizedModuleLoader.register}
    */
-  register(specifier, parentURL, data, transferList) {
+  register(specifier, parentURL, data, transferList, isInternal) {
     if (!this.#customizations) {
       // `CustomizedModuleLoader` is defined at the bottom of this file and
       // available well before this line is ever invoked. This is here in
@@ -502,7 +499,7 @@ class ModuleLoader {
       // eslint-disable-next-line no-use-before-define
       this.setCustomizations(new CustomizedModuleLoader());
     }
-    return this.#customizations.register(`${specifier}`, `${parentURL}`, data, transferList);
+    return this.#customizations.register(`${specifier}`, `${parentURL}`, data, transferList, isInternal);
   }
 
   /**
@@ -579,13 +576,18 @@ class ModuleLoader {
       this.#customizations.loadSync(url, context) :
       defaultLoadSync(url, context);
     let format = result?.format;
-    if (format === 'module') {
+    if (format === 'module' || format === 'module-typescript') {
       throw new ERR_REQUIRE_ESM(url, true);
     }
     if (format === 'commonjs') {
       format = 'require-commonjs';
       result = { __proto__: result, format };
     }
+    if (format === 'commonjs-typescript') {
+      format = 'require-commonjs-typescript';
+      result = { __proto__: result, format };
+    }
+
     this.validateLoadResult(url, format);
     return result;
   }
@@ -634,10 +636,11 @@ class CustomizedModuleLoader {
    * @param {any} [data] Arbitrary data to be passed from the custom loader
    * (user-land) to the worker.
    * @param {any[]} [transferList] Objects in `data` that are changing ownership
+   * @param {boolean} [isInternal] For internal loaders that should not be publicly exposed.
    * @returns {{ format: string, url: URL['href'] }}
    */
-  register(originalSpecifier, parentURL, data, transferList) {
-    return hooksProxy.makeSyncRequest('register', transferList, originalSpecifier, parentURL, data);
+  register(originalSpecifier, parentURL, data, transferList, isInternal) {
+    return hooksProxy.makeSyncRequest('register', transferList, originalSpecifier, parentURL, data, isInternal);
   }
 
   /**
