@@ -3,6 +3,7 @@
 const {
   ArrayPrototypeMap,
   Boolean,
+  FunctionPrototypeCall,
   JSONParse,
   ObjectKeys,
   ObjectPrototypeHasOwnProperty,
@@ -37,12 +38,14 @@ const { readFileSync } = require('fs');
 const { dirname, extname, isAbsolute } = require('path');
 const {
   loadBuiltinModule,
+  stripTypeScriptTypes,
   stripBOM,
   urlToFilename,
 } = require('internal/modules/helpers');
 const {
   kIsCachedByESMLoader,
   Module: CJSModule,
+  wrapModuleLoad,
   kModuleSource,
   kModuleExport,
   kModuleExportNames,
@@ -51,7 +54,7 @@ const { fileURLToPath, pathToFileURL, URL } = require('internal/url');
 let debug = require('internal/util/debuglog').debuglog('esm', (fn) => {
   debug = fn;
 });
-const { emitExperimentalWarning, kEmptyObject, setOwnProperty } = require('internal/util');
+const { emitExperimentalWarning, kEmptyObject, setOwnProperty, isWindows } = require('internal/util');
 const {
   ERR_UNKNOWN_BUILTIN_MODULE,
   ERR_INVALID_RETURN_PROPERTY_VALUE,
@@ -176,15 +179,14 @@ translators.set('module', function moduleStrategy(url, source, isMain) {
  * @param {boolean} isMain - Whether the module is the entrypoint
  */
 function loadCJSModule(module, source, url, filename, isMain) {
-  const compileResult = compileFunctionForCJSLoader(source, filename, isMain, false);
+  const compileResult = compileFunctionForCJSLoader(source, filename, false /* is_sea_main */, false);
 
+  const { function: compiledWrapper, sourceMapURL } = compileResult;
   // Cache the source map for the cjs module if present.
-  if (compileResult.sourceMapURL) {
-    maybeCacheSourceMap(url, source, null, false, undefined, compileResult.sourceMapURL);
+  if (sourceMapURL) {
+    maybeCacheSourceMap(url, source, module, false, undefined, sourceMapURL);
   }
 
-  const compiledWrapper = compileResult.function;
-
   const cascadedLoader = require('internal/modules/esm/loader').getOrInitializeCascadedLoader();
   const __dirname = dirname(filename);
   // eslint-disable-next-line func-name-matching,func-style
@@ -198,7 +200,7 @@ function loadCJSModule(module, source, url, filename, isMain) {
           importAttributes = { __proto__: null, type: 'json' };
           break;
         case '.node':
-          return CJSModule._load(specifier, module);
+          return wrapModuleLoad(specifier, module);
         default:
             // fall through
       }
@@ -290,7 +292,7 @@ translators.set('commonjs-sync', function requireCommonJS(url, source, isMain) {
   return createCJSModuleWrap(url, source, isMain, (module, source, url, filename, isMain) => {
     assert(module === CJSModule._cache[filename]);
     assert(!isMain);
-    CJSModule._load(filename, null, isMain);
+    wrapModuleLoad(filename, null, isMain);
   });
 });
 
@@ -302,6 +304,15 @@ translators.set('require-commonjs', (url, source, isMain) => {
   return createCJSModuleWrap(url, source);
 });
 
+// Handle CommonJS modules referenced by `require` calls.
+// This translator function must be sync, as `require` is sync.
+translators.set('require-commonjs-typescript', (url, source, isMain) => {
+  emitExperimentalWarning('Type Stripping');
+  assert(cjsParse);
+  const code = stripTypeScriptTypes(stringify(source), url);
+  return createCJSModuleWrap(url, code);
+});
+
 // Handle CommonJS modules referenced by `import` statements or expressions,
 // or as the initial entry point when the ESM loader handles a CommonJS entry.
 translators.set('commonjs', async function commonjsStrategy(url, source,
@@ -315,7 +326,7 @@ translators.set('commonjs', async function commonjsStrategy(url, source,
   // obtained by calling the monkey-patchable CJS loader.
   const cjsLoader = source == null ? (module, source, url, filename, isMain) => {
     assert(module === CJSModule._cache[filename]);
-    CJSModule._load(filename, undefined, isMain);
+    wrapModuleLoad(filename, undefined, isMain);
   } : loadCJSModule;
 
   try {
@@ -416,7 +427,6 @@ translators.set('builtin', function builtinStrategy(url) {
 });
 
 // Strategy for loading a JSON file
-const isWindows = process.platform === 'win32';
 translators.set('json', function jsonStrategy(url, source) {
   emitExperimentalWarning('Importing JSON modules');
   assertBufferSource(source, true, 'load');
@@ -511,3 +521,21 @@ translators.set('wasm', async function(url, source) {
     }
   }).module;
 });
+
+// Strategy for loading a commonjs TypeScript module
+translators.set('commonjs-typescript', function(url, source) {
+  emitExperimentalWarning('Type Stripping');
+  assertBufferSource(source, false, 'load');
+  const code = stripTypeScriptTypes(stringify(source), url);
+  debug(`Translating TypeScript ${url}`);
+  return FunctionPrototypeCall(translators.get('commonjs'), this, url, code, false);
+});
+
+// Strategy for loading an esm TypeScript module
+translators.set('module-typescript', function(url, source) {
+  emitExperimentalWarning('Type Stripping');
+  assertBufferSource(source, false, 'load');
+  const code = stripTypeScriptTypes(stringify(source), url);
+  debug(`Translating TypeScript ${url}`);
+  return FunctionPrototypeCall(translators.get('module'), this, url, code, false);
+});
