@@ -46,10 +46,11 @@ const {
   validateNumber,
   validateObject,
 } = require('internal/validators');
-const { getValidatedPath } = require('internal/fs/utils');
-const { toNamespacedPath } = require('path');
+
 const constants = internalBinding('constants').os.signals;
 
+let getValidatedPath; // We need to lazy load it because of the circular dependency.
+
 const kInternal = Symbol('internal properties');
 
 function assert(x, msg) {
@@ -254,14 +255,14 @@ function wrapProcessMethods(binding) {
    */
   function loadEnvFile(path = undefined) { // Provide optional value so that `loadEnvFile.length` returns 0
     if (path != null) {
+      getValidatedPath ??= require('internal/fs/utils').getValidatedPath;
       path = getValidatedPath(path);
-      _loadEnvFile(toNamespacedPath(path));
+      _loadEnvFile(path);
     } else {
       _loadEnvFile();
     }
   }
 
-
   return {
     _rawDebug,
     cpuUsage,
@@ -284,7 +285,8 @@ function buildAllowedFlags() {
     envSettings: { kAllowedInEnvvar },
     types: { kBoolean },
   } = internalBinding('options');
-  const { options, aliases } = require('internal/options');
+  const { getCLIOptionsInfo } = require('internal/options');
+  const { options, aliases } = getCLIOptionsInfo();
 
   const allowedNodeEnvironmentFlags = [];
   for (const { 0: name, 1: info } of options) {
@@ -422,6 +424,8 @@ function toggleTraceCategoryState(asyncHooksEnabled) {
   }
 }
 
+const { arch, platform, version } = process;
+
 module.exports = {
   toggleTraceCategoryState,
   assert,
@@ -429,4 +433,7 @@ module.exports = {
   wrapProcessMethods,
   hrtime,
   hrtimeBigInt,
+  arch,
+  platform,
+  version,
 };
