@@ -32,6 +32,7 @@ const {
   exposeLazyInterfaces,
   defineReplaceableLazyAttribute,
   setupCoverageHooks,
+  emitExperimentalWarning,
 } = require('internal/util');
 
 const {
@@ -102,10 +103,12 @@ function prepareExecution(options) {
   const mainEntry = patchProcessObject(expandArgv1);
   setupTraceCategoryState();
   setupInspectorHooks();
+  setupNetworkInspection();
   setupNavigator();
   setupWarningHandler();
   setupUndici();
   setupWebCrypto();
+  setupSQLite();
   setupWebStorage();
   setupCustomEvent();
   setupEventsource();
@@ -378,6 +381,15 @@ function setupWebCrypto() {
   }
 }
 
+function setupSQLite() {
+  if (!getOptionValue('--experimental-sqlite')) {
+    return;
+  }
+
+  const { BuiltinModule } = require('internal/bootstrap/realm');
+  BuiltinModule.allowRequireByUsers('sqlite');
+}
+
 function setupWebStorage() {
   if (getEmbedderOptions().noBrowserGlobals ||
       !getOptionValue('--experimental-webstorage')) {
@@ -498,6 +510,16 @@ function setupInspectorHooks() {
   }
 }
 
+function setupNetworkInspection() {
+  if (internalBinding('config').hasInspector && getOptionValue('--experimental-network-inspection')) {
+    const {
+      enable,
+      disable,
+    } = require('internal/inspector_network_tracking');
+    internalBinding('inspector').setupNetworkTracking(enable, disable);
+  }
+}
+
 // In general deprecations are initialized wherever the APIs are implemented,
 // this is used to deprecate APIs implemented in C++ where the deprecation
 // utilities are not easily accessible.
@@ -598,8 +620,7 @@ function initializePermission() {
     };
     // Guarantee path module isn't monkey-patched to bypass permission model
     ObjectFreeze(require('path'));
-    process.emitWarning('Permission is an experimental feature',
-                        'ExperimentalWarning');
+    emitExperimentalWarning('Permission');
     const { has } = require('internal/process/permission');
     const warnFlags = [
       '--allow-addons',
@@ -688,8 +709,7 @@ function initializeSourceMapsHandlers() {
 
 function initializeFrozenIntrinsics() {
   if (getOptionValue('--frozen-intrinsics')) {
-    process.emitWarning('The --frozen-intrinsics flag is experimental',
-                        'ExperimentalWarning');
+    emitExperimentalWarning('Frozen intristics');
     require('internal/freeze_intrinsics')();
   }
 }
