@@ -32,8 +32,10 @@ const {
   ObjectDefineProperties,
   ObjectDefineProperty,
   ObjectSetPrototypeOf,
+  Promise,
   StringPrototypeToLowerCase,
   Symbol,
+  SymbolAsyncDispose,
   SymbolHasInstance,
 } = primordials;
 
@@ -44,6 +46,7 @@ const EE = require('events');
 const Stream = require('internal/streams/legacy').Stream;
 const { Buffer } = require('buffer');
 const destroyImpl = require('internal/streams/destroy');
+const eos = require('internal/streams/end-of-stream');
 
 const {
   addAbortSignal,
@@ -54,16 +57,19 @@ const {
   getDefaultHighWaterMark,
 } = require('internal/streams/state');
 const {
-  ERR_INVALID_ARG_TYPE,
-  ERR_METHOD_NOT_IMPLEMENTED,
-  ERR_MULTIPLE_CALLBACK,
-  ERR_STREAM_CANNOT_PIPE,
-  ERR_STREAM_DESTROYED,
-  ERR_STREAM_ALREADY_FINISHED,
-  ERR_STREAM_NULL_VALUES,
-  ERR_STREAM_WRITE_AFTER_END,
-  ERR_UNKNOWN_ENCODING,
-} = require('internal/errors').codes;
+  AbortError,
+  codes: {
+    ERR_INVALID_ARG_TYPE,
+    ERR_METHOD_NOT_IMPLEMENTED,
+    ERR_MULTIPLE_CALLBACK,
+    ERR_STREAM_ALREADY_FINISHED,
+    ERR_STREAM_CANNOT_PIPE,
+    ERR_STREAM_DESTROYED,
+    ERR_STREAM_NULL_VALUES,
+    ERR_STREAM_WRITE_AFTER_END,
+    ERR_UNKNOWN_ENCODING,
+  },
+} = require('internal/errors');
 const {
   kState,
   // bitfields
@@ -252,8 +258,8 @@ ObjectDefineProperties(WritableState.prototype, {
     enumerable: false,
     get() { return (this[kState] & kWriteCb) !== 0 ? this[kWriteCbValue] : nop; },
     set(value) {
+      this[kWriteCbValue] = value;
       if (value) {
-        this[kWriteCbValue] = value;
         this[kState] |= kWriteCb;
       } else {
         this[kState] &= ~kWriteCb;
@@ -268,8 +274,8 @@ ObjectDefineProperties(WritableState.prototype, {
     enumerable: false,
     get() { return (this[kState] & kAfterWriteTickInfo) !== 0 ? this[kAfterWriteTickInfoValue] : null; },
     set(value) {
+      this[kAfterWriteTickInfoValue] = value;
       if (value) {
-        this[kAfterWriteTickInfoValue] = value;
         this[kState] |= kAfterWriteTickInfo;
       } else {
         this[kState] &= ~kAfterWriteTickInfo;
@@ -294,7 +300,7 @@ ObjectDefineProperties(WritableState.prototype, {
 });
 
 function WritableState(options, stream, isDuplex) {
-  // Bit map field to store WritableState more effciently with 1 bit per field
+  // Bit map field to store WritableState more efficiently with 1 bit per field
   // instead of a V8 slot per field.
   this[kState] = kSync | kConstructed | kEmitClose | kAutoDestroy;
 
@@ -565,7 +571,7 @@ function writeOrBuffer(stream, state, chunk, encoding, callback) {
     state[kState] &= ~kSync;
   }
 
-  const ret = state.length < state.highWaterMark;
+  const ret = state.length < state.highWaterMark || state.length === 0;
 
   if (!ret) {
     state[kState] |= kNeedDrain;
@@ -615,7 +621,8 @@ function onwrite(stream, er) {
   const sync = (state[kState] & kSync) !== 0;
   const cb = (state[kState] & kWriteCb) !== 0 ? state[kWriteCbValue] : nop;
 
-  state[kState] &= ~(kWriting | kExpectWriteCb | kWriteCb);
+  state.writecb = null;
+  state[kState] &= ~(kWriting | kExpectWriteCb);
   state.length -= state.writelen;
   state.writelen = 0;
 
@@ -1004,8 +1011,8 @@ ObjectDefineProperties(Writable.prototype, {
       // where the writable side was disabled upon construction.
       // Compat. The user might manually disable writable side through
       // deprecated setter.
-      return !!w && w.writable !== false && !w.errored &&
-        (w[kState] & (kEnding | kEnded | kDestroyed)) === 0;
+      return !!w && w.writable !== false &&
+        (w[kState] & (kEnding | kEnded | kDestroyed | kErrored)) === 0;
     },
     set(val) {
       // Backwards compatible.
@@ -1141,3 +1148,14 @@ Writable.fromWeb = function(writableStream, options) {
 Writable.toWeb = function(streamWritable) {
   return lazyWebStreams().newWritableStreamFromStreamWritable(streamWritable);
 };
+
+Writable.prototype[SymbolAsyncDispose] = function() {
+  let error;
+  if (!this.destroyed) {
+    error = this.writableFinished ? null : new AbortError();
+    this.destroy(error);
+  }
+  return new Promise((resolve, reject) =>
+    eos(this, (err) => (err && err.name !== 'AbortError' ? reject(err) : resolve(null))),
+  );
+};
