@@ -26,7 +26,7 @@ const {
 } = require('fs');
 const { setupCoverageHooks } = require('internal/util');
 const { tmpdir } = require('os');
-const { join, resolve } = require('path');
+const { join, resolve, relative, matchesGlob } = require('path');
 const { fileURLToPath } = require('internal/url');
 const { kMappings, SourceMap } = require('internal/source_map/source_map');
 const kCoverageFileRegex = /^coverage-(\d+)-(\d{13})-(\d+)\.json$/;
@@ -50,10 +50,12 @@ class CoverageLine {
 }
 
 class TestCoverage {
-  constructor(coverageDirectory, originalCoverageDirectory, workingDirectory) {
+  constructor(coverageDirectory, originalCoverageDirectory, workingDirectory, excludeGlobs, includeGlobs) {
     this.coverageDirectory = coverageDirectory;
     this.originalCoverageDirectory = originalCoverageDirectory;
     this.workingDirectory = workingDirectory;
+    this.excludeGlobs = excludeGlobs;
+    this.includeGlobs = includeGlobs;
   }
 
   #sourceLines = new SafeMap();
@@ -308,7 +310,7 @@ class TestCoverage {
 
         const coverageFile = join(this.coverageDirectory, entry.name);
         const coverage = JSONParse(readFileSync(coverageFile, 'utf8'));
-        mergeCoverage(result, this.mapCoverageWithSourceMap(coverage));
+        this.mergeCoverage(result, this.mapCoverageWithSourceMap(coverage));
       }
 
       return ArrayFrom(result.values());
@@ -331,15 +333,15 @@ class TestCoverage {
       const script = result[i];
       const { url, functions } = script;
 
-      if (shouldSkipFileCoverage(url) || sourceMapCache[url] == null) {
+      if (this.shouldSkipFileCoverage(url) || sourceMapCache[url] == null) {
         newResult.set(url, script);
         continue;
       }
       const { data, lineLengths } = sourceMapCache[url];
       let offset = 0;
       const executedLines = ArrayPrototypeMap(lineLengths, (length, i) => {
-        const coverageLine = new CoverageLine(i + 1, offset, null, length);
-        offset += length;
+        const coverageLine = new CoverageLine(i + 1, offset, null, length + 1);
+        offset += length + 1;
         return coverageLine;
       });
       if (data.sourcesContent != null) {
@@ -407,6 +409,54 @@ class TestCoverage {
     return MathMin(lines[line].startOffset + entry.originalColumn, lines[line].endOffset);
   }
 
+  mergeCoverage(merged, coverage) {
+    for (let i = 0; i < coverage.length; ++i) {
+      const newScript = coverage[i];
+      const { url } = newScript;
+
+      if (this.shouldSkipFileCoverage(url)) {
+        continue;
+      }
+
+      const oldScript = merged.get(url);
+
+      if (oldScript === undefined) {
+        merged.set(url, newScript);
+      } else {
+        mergeCoverageScripts(oldScript, newScript);
+      }
+    }
+  }
+
+  shouldSkipFileCoverage(url) {
+    // This check filters out core modules, which start with 'node:' in
+    // coverage reports, as well as any invalid coverages which have been
+    // observed on Windows.
+    if (!StringPrototypeStartsWith(url, 'file:')) return true;
+
+    const absolutePath = fileURLToPath(url);
+    const relativePath = relative(this.workingDirectory, absolutePath);
+
+    // This check filters out files that match the exclude globs.
+    if (this.excludeGlobs?.length > 0) {
+      for (let i = 0; i < this.excludeGlobs.length; ++i) {
+        if (matchesGlob(relativePath, this.excludeGlobs[i]) ||
+            matchesGlob(absolutePath, this.excludeGlobs[i])) return true;
+      }
+    }
+
+    // This check filters out files that do not match the include globs.
+    if (this.includeGlobs?.length > 0) {
+      for (let i = 0; i < this.includeGlobs.length; ++i) {
+        if (matchesGlob(relativePath, this.includeGlobs[i]) ||
+            matchesGlob(absolutePath, this.includeGlobs[i])) return false;
+      }
+      return true;
+    }
+
+    // This check filters out the node_modules/ directory, unless it is explicitly included.
+    return StringPrototypeIncludes(url, '/node_modules/');
+  }
 }
 
 function toPercentage(covered, total) {
@@ -417,7 +467,7 @@ function sortCoverageFiles(a, b) {
   return StringPrototypeLocaleCompare(a.path, b.path);
 }
 
-function setupCoverage() {
+function setupCoverage(options) {
   let originalCoverageDirectory = process.env.NODE_V8_COVERAGE;
   const cwd = process.cwd();
 
@@ -441,7 +491,13 @@ function setupCoverage() {
   // child processes.
   process.env.NODE_V8_COVERAGE = coverageDirectory;
 
-  return new TestCoverage(coverageDirectory, originalCoverageDirectory, cwd);
+  return new TestCoverage(
+    coverageDirectory,
+    originalCoverageDirectory,
+    cwd,
+    options.coverageExcludeGlobs,
+    options.coverageIncludeGlobs,
+  );
 }
 
 function mapRangeToLines(range, lines) {
@@ -485,35 +541,6 @@ function mapRangeToLines(range, lines) {
   return { __proto__: null, lines: mappedLines, ignoredLines };
 }
 
-function shouldSkipFileCoverage(url) {
-  // The first part of this check filters out the node_modules/ directory
-  // from the results. This filter is applied first because most real world
-  // applications will be dominated by third party dependencies. The second
-  // part of the check filters out core modules, which start with 'node:' in
-  // coverage reports, as well as any invalid coverages which have been
-  // observed on Windows.
-  return StringPrototypeIncludes(url, '/node_modules/') || !StringPrototypeStartsWith(url, 'file:');
-}
-
-function mergeCoverage(merged, coverage) {
-  for (let i = 0; i < coverage.length; ++i) {
-    const newScript = coverage[i];
-    const { url } = newScript;
-
-    if (shouldSkipFileCoverage(url)) {
-      continue;
-    }
-
-    const oldScript = merged.get(url);
-
-    if (oldScript === undefined) {
-      merged.set(url, newScript);
-    } else {
-      mergeCoverageScripts(oldScript, newScript);
-    }
-  }
-}
-
 function mergeCoverageScripts(oldScript, newScript) {
   // Merge the functions from the new coverage into the functions from the
   // existing (merged) coverage.
