@@ -34,7 +34,13 @@ const {
 } = require('internal/errors');
 const esmLoader = require('internal/modules/esm/loader');
 const { getOptionValue } = require('internal/options');
-const { fileURLToPath, toPathIfFileURL, URL } = require('internal/url');
+const {
+  fileURLToPath,
+  isURL,
+  pathToFileURL,
+  toPathIfFileURL,
+  URL,
+} = require('internal/url');
 const {
   emitExperimentalWarning,
   getStructuredStack,
@@ -49,7 +55,6 @@ const {
   validateInteger,
   validateObject,
   validateOneOf,
-  validateString,
 } = require('internal/validators');
 const { MockTimers } = require('internal/test_runner/mock/mock_timers');
 const { strictEqual, notStrictEqual } = require('assert');
@@ -488,7 +493,11 @@ class MockTracker {
 
   module(specifier, options = kEmptyObject) {
     emitExperimentalWarning('Module mocking');
-    validateString(specifier, 'specifier');
+    if (typeof specifier !== 'string') {
+      if (!isURL(specifier))
+        throw new ERR_INVALID_ARG_TYPE('specifier', ['string', 'URL'], specifier);
+      specifier = `${specifier}`;
+    }
     validateObject(options, 'options');
     debug('module mock entry, specifier = "%s", options = %o', specifier, options);
 
@@ -508,7 +517,7 @@ class MockTracker {
 
     // Get the file that called this function. We need four stack frames:
     // vm context -> getStructuredStack() -> this function -> actual caller.
-    const caller = getStructuredStack()[3]?.getFileName();
+    const caller = pathToFileURL(getStructuredStack()[3]?.getFileName()).href;
     const { format, url } = sharedState.moduleLoader.resolveSync(
       mockSpecifier, caller, null,
     );
@@ -526,7 +535,7 @@ class MockTracker {
 
     if (baseURL.searchParams.has(kMockSearchParam)) {
       throw new ERR_INVALID_STATE(
-        `Cannot mock '${specifier}'. The module is already mocked.`,
+        `Cannot mock '${specifier}.' The module is already mocked.`,
       );
     }
 
