@@ -1,4 +1,5 @@
 'use strict';
+
 const {
   ArrayIsArray,
   ArrayPrototypeEvery,
@@ -24,6 +25,7 @@ const {
   StringPrototypeIndexOf,
   StringPrototypeSlice,
   StringPrototypeStartsWith,
+  Symbol,
   TypedArrayPrototypeGetLength,
   TypedArrayPrototypeSubarray,
 } = primordials;
@@ -32,8 +34,7 @@ const { spawn } = require('child_process');
 const { finished } = require('internal/streams/end-of-stream');
 const { resolve } = require('path');
 const { DefaultDeserializer, DefaultSerializer } = require('v8');
-// TODO(aduh95): switch to internal/readline/interface when backporting to Node.js 16.x is no longer a concern.
-const { createInterface } = require('readline');
+const { Interface } = require('internal/readline/interface');
 const { deserializeError } = require('internal/error_serdes');
 const { Buffer } = require('buffer');
 const { FilesWatcher } = require('internal/watch_mode/files_watcher');
@@ -45,18 +46,28 @@ const {
     ERR_TEST_FAILURE,
   },
 } = require('internal/errors');
+const esmLoader = require('internal/modules/esm/loader');
 const {
   validateArray,
   validateBoolean,
   validateFunction,
   validateObject,
+  validateOneOf,
   validateInteger,
 } = require('internal/validators');
 const { getInspectPort, isUsingInspector, isInspectorMessage } = require('internal/util/inspector');
 const { isRegExp } = require('internal/util/types');
-const { kEmptyObject } = require('internal/util');
+const { pathToFileURL } = require('internal/url');
+const {
+  createDeferredPromise,
+  getCWDURL,
+  kEmptyObject,
+} = require('internal/util');
 const { kEmitMessage } = require('internal/test_runner/tests_stream');
-const { createTestTree } = require('internal/test_runner/harness');
+const {
+  createTestTree,
+  startSubtestAfterBootstrap,
+} = require('internal/test_runner/harness');
 const {
   kAborted,
   kCancelledByParent,
@@ -70,7 +81,7 @@ const {
   convertStringToRegExp,
   countCompletedTest,
   kDefaultPattern,
-  shouldColorizeTestFiles,
+  parseCommandLine,
 } = require('internal/test_runner/utils');
 const { Glob } = require('internal/fs/glob');
 const { once } = require('events');
@@ -78,7 +89,11 @@ const {
   triggerUncaughtException,
   exitCodes: { kGenericUserError },
 } = internalBinding('errors');
+let debug = require('internal/util/debuglog').debuglog('test_runner', (fn) => {
+  debug = fn;
+});
 
+const kIsolatedProcessName = Symbol('kIsolatedProcessName');
 const kFilterArgs = ['--test', '--experimental-test-coverage', '--watch'];
 const kFilterArgValues = ['--test-reporter', '--test-reporter-destination'];
 const kDiagnosticsFilterArgs = ['tests', 'suites', 'pass', 'fail', 'cancelled', 'skipped', 'todo', 'duration_ms'];
@@ -88,10 +103,11 @@ const kCanceledTests = new SafeSet()
 
 let kResistStopPropagation;
 
-function createTestFileList() {
-  const cwd = process.cwd();
-  const hasUserSuppliedPattern = process.argv.length > 1;
-  const patterns = hasUserSuppliedPattern ? ArrayPrototypeSlice(process.argv, 1) : [kDefaultPattern];
+function createTestFileList(patterns, cwd) {
+  const hasUserSuppliedPattern = patterns != null;
+  if (!patterns || patterns.length === 0) {
+    patterns = [kDefaultPattern];
+  }
   const glob = new Glob(patterns, {
     __proto__: null,
     cwd,
@@ -129,7 +145,12 @@ function getRunArgs(path, { forceExit, inspectPort, testNamePatterns, testSkipPa
   if (only === true) {
     ArrayPrototypePush(argv, '--test-only');
   }
-  ArrayPrototypePush(argv, path);
+
+  if (path === kIsolatedProcessName) {
+    ArrayPrototypePush(argv, '--test', ...ArrayPrototypeSlice(process.argv, 1));
+  } else {
+    ArrayPrototypePush(argv, path);
+  }
 
   return argv;
 }
@@ -325,7 +346,9 @@ class FileTest extends Test {
 
 function runTestFile(path, filesWatcher, opts) {
   const watchMode = filesWatcher != null;
-  const subtest = opts.root.createSubtest(FileTest, path, { __proto__: null, signal: opts.signal }, async (t) => {
+  const testPath = path === kIsolatedProcessName ? '' : path;
+  const testOpts = { __proto__: null, signal: opts.signal };
+  const subtest = opts.root.createSubtest(FileTest, testPath, testOpts, async (t) => {
     const args = getRunArgs(path, opts);
     const stdio = ['pipe', 'pipe', 'pipe'];
     const env = { __proto__: null, ...process.env, NODE_TEST_CONTEXT: 'child-v8' };
@@ -337,7 +360,17 @@ function runTestFile(path, filesWatcher, opts) {
       env.FORCE_COLOR = '1';
     }
 
-    const child = spawn(process.execPath, args, { __proto__: null, signal: t.signal, encoding: 'utf8', env, stdio });
+    const child = spawn(
+      process.execPath, args,
+      {
+        __proto__: null,
+        signal: t.signal,
+        encoding: 'utf8',
+        env,
+        stdio,
+        cwd: opts.cwd,
+      },
+    );
     if (watchMode) {
       filesWatcher.runningProcesses.set(path, child);
       filesWatcher.watcher.watchChildProcessModules(child, path);
@@ -345,7 +378,6 @@ function runTestFile(path, filesWatcher, opts) {
 
     let err;
 
-
     child.on('error', (error) => {
       err = error;
     });
@@ -354,7 +386,7 @@ function runTestFile(path, filesWatcher, opts) {
       subtest.parseMessage(data);
     });
 
-    const rl = createInterface({ __proto__: null, input: child.stderr });
+    const rl = new Interface({ __proto__: null, input: child.stderr });
     rl.on('line', (line) => {
       if (isInspectorMessage(line)) {
         process.stderr.write(line + '\n');
@@ -414,45 +446,64 @@ function runTestFile(path, filesWatcher, opts) {
 function watchFiles(testFiles, opts) {
   const runningProcesses = new SafeMap();
   const runningSubtests = new SafeMap();
-  const watcher = new FilesWatcher({ __proto__: null, debounce: 200, mode: 'filter', signal: opts.signal });
+  const watcherMode = opts.hasFiles ? 'filter' : 'all';
+  const watcher = new FilesWatcher({ __proto__: null, debounce: 200, mode: watcherMode, signal: opts.signal });
+  if (!opts.hasFiles) {
+    watcher.watchPath(opts.cwd);
+  }
   const filesWatcher = { __proto__: null, watcher, runningProcesses, runningSubtests };
   opts.root.harness.watching = true;
 
-  watcher.on('changed', ({ owners, eventType }) => {
-    if (eventType === 'rename') {
-      const updatedTestFiles = createTestFileList();
+  async function restartTestFile(file) {
+    const runningProcess = runningProcesses.get(file);
+    if (runningProcess) {
+      runningProcess.kill();
+      await once(runningProcess, 'exit');
+    }
+    if (!runningSubtests.size) {
+      // Reset the topLevel counter
+      opts.root.harness.counters.topLevel = 0;
+    }
+    await runningSubtests.get(file);
+    runningSubtests.set(file, runTestFile(file, filesWatcher, opts));
+  }
 
+  // Watch for changes in current filtered files
+  watcher.on('changed', ({ owners, eventType }) => {
+    if (!opts.hasFiles && (eventType === 'rename' || eventType === 'change')) {
+      const updatedTestFiles = createTestFileList(opts.globPatterns, opts.cwd);
       const newFileName = ArrayPrototypeFind(updatedTestFiles, (x) => !ArrayPrototypeIncludes(testFiles, x));
       const previousFileName = ArrayPrototypeFind(testFiles, (x) => !ArrayPrototypeIncludes(updatedTestFiles, x));
 
-      // When file renamed
-      if (newFileName && previousFileName) {
+      testFiles = updatedTestFiles;
+
+      // When file renamed (created / deleted) we need to update the watcher
+      if (newFileName) {
         owners = new SafeSet().add(newFileName);
         watcher.filterFile(resolve(newFileName), owners);
       }
 
-      testFiles = updatedTestFiles;
+      if (!newFileName && previousFileName) {
+        return; // Avoid rerunning files when file deleted
+      }
     }
 
-    watcher.unfilterFilesOwnedBy(owners);
-    PromisePrototypeThen(SafePromiseAllReturnVoid(testFiles, async (file) => {
-      if (!owners.has(file)) {
-        return;
-      }
-      const runningProcess = runningProcesses.get(file);
-      if (runningProcess) {
-        runningProcess.kill();
-        await once(runningProcess, 'exit');
-      }
-      if (!runningSubtests.size) {
-        // Reset the topLevel counter
-        opts.root.harness.counters.topLevel = 0;
-      }
-      await runningSubtests.get(file);
-      runningSubtests.set(file, runTestFile(file, filesWatcher, opts));
-    }, undefined, (error) => {
-      triggerUncaughtException(error, true /* fromPromise */);
-    }));
+    if (opts.isolation === 'none') {
+      PromisePrototypeThen(restartTestFile(kIsolatedProcessName), undefined, (error) => {
+        triggerUncaughtException(error, true /* fromPromise */);
+      });
+    } else {
+      watcher.unfilterFilesOwnedBy(owners);
+      PromisePrototypeThen(SafePromiseAllReturnVoid(testFiles, async (file) => {
+        if (!owners.has(file)) {
+          return;
+        }
+
+        await restartTestFile(file);
+      }, undefined, (error) => {
+        triggerUncaughtException(error, true /* fromPromise */);
+      }));
+    }
   });
   if (opts.signal) {
     kResistStopPropagation ??= require('internal/event_target').kResistStopPropagation;
@@ -480,10 +531,11 @@ function run(options = kEmptyObject) {
     files,
     forceExit,
     inspectPort,
+    isolation = 'process',
     watch,
     setup,
     only,
-    plan,
+    globPatterns,
   } = options;
 
   if (files != null) {
@@ -504,6 +556,16 @@ function run(options = kEmptyObject) {
   if (only != null) {
     validateBoolean(only, 'options.only');
   }
+  if (globPatterns != null) {
+    validateArray(globPatterns, 'options.globPatterns');
+  }
+
+  if (globPatterns?.length > 0 && files?.length > 0) {
+    throw new ERR_INVALID_ARG_VALUE(
+      'options.globPatterns', globPatterns, 'is not supported when specifying \'options.files\'',
+    );
+  }
+
   if (shard != null) {
     validateObject(shard, 'options.shard');
     // Avoid re-evaluating the shard object in case it's a getter
@@ -551,23 +613,31 @@ function run(options = kEmptyObject) {
       throw new ERR_INVALID_ARG_TYPE(name, ['string', 'RegExp'], value);
     });
   }
+  validateOneOf(isolation, 'options.isolation', ['process', 'none']);
 
-  const root = createTestTree({ __proto__: null, concurrency, timeout, signal, plan });
-  root.harness.shouldColorizeTestFiles ||= shouldColorizeTestFiles(root);
+  const rootTestOptions = { __proto__: null, concurrency, timeout, signal };
+  const globalOptions = {
+    __proto__: null,
+    // parseCommandLine() should not be used here. However, The existing run()
+    // behavior has relied on it, so removing it must be done in a semver major.
+    ...parseCommandLine(),
+    setup,  // This line can be removed when parseCommandLine() is removed here.
+  };
+  const root = createTestTree(rootTestOptions, globalOptions);
 
-  if (process.env.NODE_TEST_CONTEXT !== undefined) {
-    process.emitWarning('node:test run() is being called recursively within a test file. skipping running files.');
-    root.postRun();
-    return root.reporter;
-  }
-  let testFiles = files ?? createTestFileList();
+  // This const should be replaced by a run option in the future.
+  const cwd = process.cwd();
+
+  let testFiles = files ?? createTestFileList(globPatterns, cwd);
 
   if (shard) {
     testFiles = ArrayPrototypeFilter(testFiles, (_, index) => index % shard.total === shard.index - 1);
   }
 
-  let postRun = () => root.postRun();
+  let teardown;
+  let postRun;
   let filesWatcher;
+  let runFiles;
   const opts = {
     __proto__: null,
     root,
@@ -575,24 +645,103 @@ function run(options = kEmptyObject) {
     inspectPort,
     testNamePatterns,
     testSkipPatterns,
+    hasFiles: files != null,
+    globPatterns,
     only,
     forceExit,
-  };
-  if (watch) {
-    filesWatcher = watchFiles(testFiles, opts);
-    postRun = undefined;
-  }
-  const runFiles = () => {
-    root.harness.bootstrapComplete = true;
-    root.harness.allowTestsToRun = true;
-    return SafePromiseAllSettledReturnVoid(testFiles, (path) => {
-      const subtest = runTestFile(path, filesWatcher, opts);
-      filesWatcher?.runningSubtests.set(path, subtest);
-      return subtest;
-    });
+    cwd,
+    isolation,
   };
 
-  PromisePrototypeThen(PromisePrototypeThen(PromiseResolve(setup?.(root.reporter)), runFiles), postRun);
+  if (isolation === 'process') {
+    if (process.env.NODE_TEST_CONTEXT !== undefined) {
+      process.emitWarning('node:test run() is being called recursively within a test file. skipping running files.');
+      root.postRun();
+      return root.reporter;
+    }
+
+    if (watch) {
+      filesWatcher = watchFiles(testFiles, opts);
+    } else {
+      postRun = () => root.postRun();
+      teardown = () => root.harness.teardown();
+    }
+
+    runFiles = () => {
+      root.harness.bootstrapPromise = null;
+      root.harness.buildPromise = null;
+      return SafePromiseAllSettledReturnVoid(testFiles, (path) => {
+        const subtest = runTestFile(path, filesWatcher, opts);
+        filesWatcher?.runningSubtests.set(path, subtest);
+        return subtest;
+      });
+    };
+  } else if (isolation === 'none') {
+    if (watch) {
+      filesWatcher = watchFiles(testFiles, opts);
+      runFiles = async () => {
+        root.harness.bootstrapPromise = null;
+        root.harness.buildPromise = null;
+        const subtest = runTestFile(kIsolatedProcessName, filesWatcher, opts);
+        filesWatcher?.runningSubtests.set(kIsolatedProcessName, subtest);
+        return subtest;
+      };
+    } else {
+      runFiles = async () => {
+        const { promise, resolve: finishBootstrap } = createDeferredPromise();
+
+        await root.runInAsyncScope(async () => {
+          const parentURL = getCWDURL().href;
+          const cascadedLoader = esmLoader.getOrInitializeCascadedLoader();
+          let topLevelTestCount = 0;
+
+          root.harness.bootstrapPromise = promise;
+
+          for (let i = 0; i < testFiles.length; ++i) {
+            const testFile = testFiles[i];
+            const fileURL = pathToFileURL(testFile);
+            const parent = i === 0 ? undefined : parentURL;
+            let threw = false;
+            let importError;
+
+            root.entryFile = resolve(testFile);
+            debug('loading test file:', fileURL.href);
+            try {
+              await cascadedLoader.import(fileURL, parent, { __proto__: null });
+            } catch (err) {
+              threw = true;
+              importError = err;
+            }
+
+            debug(
+              'loaded "%s": top level test count before = %d and after = %d',
+              testFile,
+              topLevelTestCount,
+              root.subtests.length,
+            );
+            if (topLevelTestCount === root.subtests.length) {
+              // This file had no tests in it. Add the placeholder test.
+              const subtest = root.createSubtest(Test, testFile);
+              if (threw) {
+                subtest.fail(importError);
+              }
+              startSubtestAfterBootstrap(subtest);
+            }
+
+            topLevelTestCount = root.subtests.length;
+          }
+        });
+
+        debug('beginning test execution');
+        root.entryFile = null;
+        finishBootstrap();
+        root.processPendingSubtests();
+      };
+    }
+  }
+
+  const setupPromise = PromiseResolve(setup?.(root.reporter));
+  PromisePrototypeThen(PromisePrototypeThen(PromisePrototypeThen(setupPromise, runFiles), postRun), teardown);
 
   return root.reporter;
 }
