@@ -11,8 +11,8 @@ const {
   FunctionPrototype,
   MathMax,
   Number,
+  NumberPrototypeToFixed,
   ObjectDefineProperty,
-  ObjectEntries,
   ObjectSeal,
   PromisePrototypeThen,
   PromiseResolve,
@@ -29,6 +29,7 @@ const {
   SymbolDispose,
 } = primordials;
 const { getCallerLocation } = internalBinding('util');
+const { exitCodes: { kGenericUserError } } = internalBinding('errors');
 const { addAbortListener } = require('internal/events/abort_listener');
 const { queueMicrotask } = require('internal/process/task_queues');
 const { AsyncResource } = require('async_hooks');
@@ -46,7 +47,6 @@ const {
   createDeferredCallback,
   countCompletedTest,
   isTestFailureError,
-  parseCommandLine,
 } = require('internal/test_runner/utils');
 const {
   createDeferredPromise,
@@ -76,18 +76,10 @@ const kHookFailure = 'hookFailed';
 const kDefaultTimeout = null;
 const noop = FunctionPrototype;
 const kShouldAbort = Symbol('kShouldAbort');
-const kFilename = process.argv?.[1];
 const kHookNames = ObjectSeal(['before', 'after', 'beforeEach', 'afterEach']);
 const kUnwrapErrors = new SafeSet()
   .add(kTestCodeFailure).add(kHookFailure)
   .add('uncaughtException').add('unhandledRejection');
-const {
-  forceExit,
-  sourceMaps,
-  testNamePatterns,
-  testSkipPatterns,
-  testOnlyFlag,
-} = parseCommandLine();
 let kResistStopPropagation;
 let assertObj;
 let findSourceMap;
@@ -103,14 +95,38 @@ function lazyFindSourceMap(file) {
   return findSourceMap(file);
 }
 
-function lazyAssertObject() {
+function lazyAssertObject(harness) {
   if (assertObj === undefined) {
     assertObj = new SafeMap();
     const assert = require('assert');
-    for (const { 0: key, 1: value } of ObjectEntries(assert)) {
-      if (typeof value === 'function') {
-        assertObj.set(value, key);
-      }
+    const methodsToCopy = [
+      'deepEqual',
+      'deepStrictEqual',
+      'doesNotMatch',
+      'doesNotReject',
+      'doesNotThrow',
+      'equal',
+      'fail',
+      'ifError',
+      'match',
+      'notDeepEqual',
+      'notDeepStrictEqual',
+      'notEqual',
+      'notStrictEqual',
+      'ok',
+      'rejects',
+      'strictEqual',
+      'throws',
+    ];
+    for (let i = 0; i < methodsToCopy.length; i++) {
+      assertObj.set(methodsToCopy[i], assert[methodsToCopy[i]]);
+    }
+
+    const { getOptionValue } = require('internal/options');
+    if (getOptionValue('--experimental-test-snapshots')) {
+      const { SnapshotManager } = require('internal/test_runner/snapshot');
+      harness.snapshotManager = new SnapshotManager(harness.config.updateSnapshots);
+      assertObj.set('snapshot', harness.snapshotManager.createAssert());
     }
   }
   return assertObj;
@@ -170,7 +186,7 @@ function testMatchesPattern(test, patterns) {
 
 class TestPlan {
   constructor(count) {
-    validateUint32(count, 'count', 0);
+    validateUint32(count, 'count');
     this.expected = count;
     this.actual = 0;
   }
@@ -201,6 +217,14 @@ class TestContext {
     return this.#test.name;
   }
 
+  get filePath() {
+    return this.#test.entryFile;
+  }
+
+  get fullName() {
+    return getFullName(this.#test);
+  }
+
   get error() {
     return this.#test.error;
   }
@@ -227,18 +251,18 @@ class TestContext {
   get assert() {
     if (this.#assert === undefined) {
       const { plan } = this.#test;
-      const assertions = lazyAssertObject();
+      const map = lazyAssertObject(this.#test.root.harness);
       const assert = { __proto__: null };
 
       this.#assert = assert;
-      for (const { 0: method, 1: name } of assertions.entries()) {
+      map.forEach((method, name) => {
         assert[name] = (...args) => {
           if (plan !== null) {
             plan.actual++;
           }
-          return ReflectApply(method, assert, args);
+          return ReflectApply(method, this, args);
         };
-      }
+      });
     }
     return this.#assert;
   }
@@ -280,23 +304,43 @@ class TestContext {
   }
 
   before(fn, options) {
-    this.#test
-      .createHook('before', fn, { __proto__: null, ...options, hookType: 'before', loc: getCallerLocation() });
+    this.#test.createHook('before', fn, {
+      __proto__: null,
+      ...options,
+      parent: this.#test,
+      hookType: 'before',
+      loc: getCallerLocation(),
+    });
   }
 
   after(fn, options) {
-    this.#test
-      .createHook('after', fn, { __proto__: null, ...options, hookType: 'after', loc: getCallerLocation() });
+    this.#test.createHook('after', fn, {
+      __proto__: null,
+      ...options,
+      parent: this.#test,
+      hookType: 'after',
+      loc: getCallerLocation(),
+    });
   }
 
   beforeEach(fn, options) {
-    this.#test
-      .createHook('beforeEach', fn, { __proto__: null, ...options, hookType: 'beforeEach', loc: getCallerLocation() });
+    this.#test.createHook('beforeEach', fn, {
+      __proto__: null,
+      ...options,
+      parent: this.#test,
+      hookType: 'beforeEach',
+      loc: getCallerLocation(),
+    });
   }
 
   afterEach(fn, options) {
-    this.#test
-      .createHook('afterEach', fn, { __proto__: null, ...options, hookType: 'afterEach', loc: getCallerLocation() });
+    this.#test.createHook('afterEach', fn, {
+      __proto__: null,
+      ...options,
+      parent: this.#test,
+      hookType: 'afterEach',
+      loc: getCallerLocation(),
+    });
   }
 }
 
@@ -314,6 +358,14 @@ class SuiteContext {
   get name() {
     return this.#suite.name;
   }
+
+  get filePath() {
+    return this.#suite.entryFile;
+  }
+
+  get fullName() {
+    return getFullName(this.#suite);
+  }
 }
 
 class Test extends AsyncResource {
@@ -325,7 +377,7 @@ class Test extends AsyncResource {
     super('Test');
 
     let { fn, name, parent } = options;
-    const { concurrency, loc, only, timeout, todo, skip, signal, plan } = options;
+    const { concurrency, entryFile, loc, only, timeout, todo, skip, signal, plan } = options;
 
     if (typeof fn !== 'function') {
       fn = noop;
@@ -347,56 +399,46 @@ class Test extends AsyncResource {
     this.filtered = false;
 
     if (parent === null) {
+      this.root = this;
+      this.harness = options.harness;
+      this.config = this.harness.config;
       this.concurrency = 1;
       this.nesting = 0;
-      this.only = testOnlyFlag;
+      this.only = this.config.only;
       this.reporter = new TestsStream();
       this.runOnlySubtests = this.only;
       this.childNumber = 0;
       this.timeout = kDefaultTimeout;
-      this.root = this;
-      this.hooks = {
-        __proto__: null,
-        before: [],
-        after: [],
-        beforeEach: [],
-        afterEach: [],
-        ownAfterEachCount: 0,
-      };
+      this.entryFile = entryFile;
     } else {
       const nesting = parent.parent === null ? parent.nesting :
         parent.nesting + 1;
 
+      this.root = parent.root;
+      this.harness = null;
+      this.config = this.root.harness.config;
       this.concurrency = parent.concurrency;
       this.nesting = nesting;
-      this.only = only ?? !parent.runOnlySubtests;
+      this.only = only ?? (parent.only && !parent.runOnlySubtests);
       this.reporter = parent.reporter;
-      this.runOnlySubtests = !this.only;
+      this.runOnlySubtests = false;
       this.childNumber = parent.subtests.length + 1;
       this.timeout = parent.timeout;
-      this.root = parent.root;
-      this.hooks = {
-        __proto__: null,
-        before: [],
-        after: [],
-        beforeEach: ArrayPrototypeSlice(parent.hooks.beforeEach),
-        afterEach: ArrayPrototypeSlice(parent.hooks.afterEach),
-        ownAfterEachCount: 0,
-      };
+      this.entryFile = parent.entryFile;
 
       if (this.willBeFiltered()) {
         this.filtered = true;
         this.parent.filteredSubtestCount++;
       }
 
-      if (testOnlyFlag && only === false) {
+      if (this.config.only && only === false) {
         fn = noop;
       }
     }
 
     switch (typeof concurrency) {
       case 'number':
-        validateUint32(concurrency, 'options.concurrency', 1);
+        validateUint32(concurrency, 'options.concurrency', true);
         this.concurrency = concurrency;
         break;
 
@@ -439,7 +481,6 @@ class Test extends AsyncResource {
     );
 
     this.fn = fn;
-    this.harness = null; // Configured on the root test by the test harness.
     this.mock = null;
     this.plan = null;
     this.expectedAssertions = plan;
@@ -459,14 +500,22 @@ class Test extends AsyncResource {
     this.subtests = [];
     this.waitingOn = 0;
     this.finished = false;
+    this.hooks = {
+      __proto__: null,
+      before: [],
+      after: [],
+      beforeEach: [],
+      afterEach: [],
+      ownAfterEachCount: 0,
+    };
 
-    if (!testOnlyFlag && (only || this.runOnlySubtests)) {
+    if (!this.config.only && (only || this.parent?.runOnlySubtests)) {
       const warning =
         "'only' and 'runOnly' require the --test-only command-line option.";
       this.diagnostic(warning);
     }
 
-    if (loc === undefined || kFilename === undefined) {
+    if (loc === undefined) {
       this.loc = undefined;
     } else {
       this.loc = {
@@ -476,7 +525,7 @@ class Test extends AsyncResource {
         file: loc[2],
       };
 
-      if (sourceMaps === true) {
+      if (this.config.sourceMaps === true) {
         const map = lazyFindSourceMap(this.loc.file);
         const entry = map?.findEntry(this.loc.line - 1, this.loc.column - 1);
 
@@ -494,7 +543,9 @@ class Test extends AsyncResource {
   }
 
   willBeFiltered() {
-    if (testOnlyFlag && !this.only) return true;
+    if (this.config.only && !this.only) return true;
+
+    const { testNamePatterns, testSkipPatterns } = this.config;
 
     if (testNamePatterns && !testMatchesPattern(this, testNamePatterns)) {
       return true;
@@ -634,6 +685,21 @@ class Test extends AsyncResource {
     this.abortController.abort();
   }
 
+  computeInheritedHooks() {
+    if (this.parent.hooks.beforeEach.length > 0) {
+      ArrayPrototypeUnshift(
+        this.hooks.beforeEach,
+        ...ArrayPrototypeSlice(this.parent.hooks.beforeEach),
+      );
+    }
+
+    if (this.parent.hooks.afterEach.length > 0) {
+      ArrayPrototypePushApply(
+        this.hooks.afterEach, ArrayPrototypeSlice(this.parent.hooks.afterEach),
+      );
+    }
+  }
+
   createHook(name, fn, options) {
     validateOneOf(name, 'hook name', kHookNames);
     // eslint-disable-next-line no-use-before-define
@@ -658,7 +724,6 @@ class Test extends AsyncResource {
     } else {
       ArrayPrototypePush(this.hooks[name], hook);
     }
-    return hook;
   }
 
   fail(err) {
@@ -709,7 +774,7 @@ class Test extends AsyncResource {
     // it. Otherwise, return a Promise to the caller and mark the test as
     // pending for later execution.
     this.reporter.enqueue(this.nesting, this.loc, this.name);
-    if (!this.root.harness.allowTestsToRun || !this.parent.hasConcurrency()) {
+    if (this.root.harness.buildPromise || !this.parent.hasConcurrency()) {
       const deferred = createDeferredPromise();
 
       deferred.test = this;
@@ -722,10 +787,7 @@ class Test extends AsyncResource {
   }
 
   [kShouldAbort]() {
-    if (this.signal.aborted) {
-      return true;
-    }
-    if (this.outerSignal?.aborted) {
+    if (this.signal.aborted || this.outerSignal?.aborted) {
       this.#abortHandler();
       return true;
     }
@@ -763,6 +825,7 @@ class Test extends AsyncResource {
   async run() {
     if (this.parent !== null) {
       this.parent.activeSubtests++;
+      this.computeInheritedHooks();
     }
     this.startTime ??= hrtime();
 
@@ -825,10 +888,7 @@ class Test extends AsyncResource {
         await SafePromiseRace([PromiseResolve(promise), stopPromise]);
       }
 
-      if (this[kShouldAbort]()) {
-        this.postRun();
-        return;
-      }
+      this[kShouldAbort]();
       this.plan?.check();
       this.pass();
       await afterEach();
@@ -864,7 +924,7 @@ class Test extends AsyncResource {
       // This helps catch any asynchronous activity that occurs after the tests
       // have finished executing.
       this.postRun();
-    } else if (forceExit) {
+    } else if (this.config.forceExit) {
       // This is the root test, and all known tests and hooks have finished
       // executing. If the user wants to force exit the process regardless of
       // any remaining ref'ed handles, then do that now. It is theoretically
@@ -935,6 +995,7 @@ class Test extends AsyncResource {
 
       // Call this harness.coverage() before collecting diagnostics, since failure to collect coverage is a diagnostic.
       const coverage = harness.coverage();
+      harness.snapshotManager?.writeSnapshotFiles();
       for (let i = 0; i < diagnostics.length; i++) {
         reporter.diagnostic(nesting, loc, diagnostics[i]);
       }
@@ -950,11 +1011,31 @@ class Test extends AsyncResource {
 
       if (coverage) {
         reporter.coverage(nesting, loc, coverage);
+
+        const coverages = [
+          { __proto__: null, actual: coverage.totals.coveredLinePercent,
+            threshold: this.config.lineCoverage, name: 'line' },
+
+          { __proto__: null, actual: coverage.totals.coveredBranchPercent,
+            threshold: this.config.branchCoverage, name: 'branch' },
+
+          { __proto__: null, actual: coverage.totals.coveredFunctionPercent,
+            threshold: this.config.functionCoverage, name: 'function' },
+        ];
+
+        for (let i = 0; i < coverages.length; i++) {
+          const { threshold, actual, name } = coverages[i];
+          if (actual < threshold) {
+            process.exitCode = kGenericUserError;
+            reporter.diagnostic(nesting, loc, `Error: ${NumberPrototypeToFixed(actual, 2)}% ${name} coverage does not meet threshold of ${threshold}%.`);
+          }
+        }
       }
 
       if (harness.watching) {
         this.reported = false;
         harness.resetCounters();
+        assertObj = undefined;
       } else {
         reporter.end();
       }
@@ -1051,14 +1132,17 @@ class Test extends AsyncResource {
 class TestHook extends Test {
   #args;
   constructor(fn, options) {
-    if (options === null || typeof options !== 'object') {
-      options = kEmptyObject;
-    }
-    const { loc, timeout, signal } = options;
-    super({ __proto__: null, fn, loc, timeout, signal });
-
-    this.parentTest = options.parent ?? null;
-    this.hookType = options.hookType;
+    const { hookType, loc, parent, timeout, signal } = options;
+    super({
+      __proto__: null,
+      fn,
+      loc,
+      timeout,
+      signal,
+      harness: parent.root.harness,
+    });
+    this.parentTest = parent;
+    this.hookType = hookType;
   }
   run(args) {
     if (this.error && !this.outerSignal?.aborted) {
@@ -1082,9 +1166,7 @@ class TestHook extends Test {
     const { error, loc, parentTest: parent } = this;
 
     // Report failures in the root test's after() hook.
-    if (error && parent !== null &&
-        parent === parent.root && this.hookType === 'after') {
-
+    if (error && parent === parent.root && this.hookType === 'after') {
       if (isTestFailureError(error)) {
         error.failureType = kHookFailure;
       }
@@ -1104,11 +1186,13 @@ class Suite extends Test {
   constructor(options) {
     super(options);
 
-    if (testNamePatterns !== null && testSkipPatterns !== null && !options.skip) {
+    if (this.config.testNamePatterns !== null &&
+        this.config.testSkipPatterns !== null &&
+        !options.skip) {
       this.fn = options.fn || this.fn;
       this.skipped = false;
     }
-    this.runOnlySubtests = testOnlyFlag;
+    this.runOnlySubtests = this.config.only;
 
     try {
       const { ctx, args } = this.getRunArgs();
@@ -1125,23 +1209,23 @@ class Suite extends Test {
       );
     } catch (err) {
       this.fail(new ERR_TEST_FAILURE(err, kTestCodeFailure));
-
-      this.buildPhaseFinished = true;
+      this.postBuild();
     }
     this.fn = noop;
   }
 
   postBuild() {
     this.buildPhaseFinished = true;
-    if (this.filtered && this.filteredSubtestCount !== this.subtests.length) {
+    if (this.filtered &&
+        (this.filteredSubtestCount !== this.subtests.length || this.error)) {
       // A suite can transition from filtered to unfiltered based on the
       // tests that it contains - in case of children matching patterns.
       this.filtered = false;
       this.parent.filteredSubtestCount--;
     } else if (
-      testOnlyFlag &&
-      testNamePatterns == null &&
-      testSkipPatterns == null &&
+      this.config.only &&
+      this.config.testNamePatterns == null &&
+      this.config.testSkipPatterns == null &&
       this.filteredSubtestCount === this.subtests.length
     ) {
       // If no subtests are marked as "only", run them all
@@ -1155,6 +1239,7 @@ class Suite extends Test {
   }
 
   async run() {
+    this.computeInheritedHooks();
     const hookArgs = this.getRunArgs();
 
     let stopPromise;
@@ -1199,6 +1284,16 @@ class Suite extends Test {
   }
 }
 
+function getFullName(test) {
+  let fullName = test.name;
+
+  for (let t = test.parent; t !== t.root; t = t.parent) {
+    fullName = `${t.name} > ${fullName}`;
+  }
+
+  return fullName;
+}
+
 module.exports = {
   kCancelledByParent,
   kSubtestsFailed,
