@@ -46,8 +46,13 @@ const {
   kEnumerableProperty,
   kEmptyObject,
   SideEffectFreeRegExpPrototypeSymbolReplace,
+  isWindows,
 } = require('internal/util');
 
+const {
+  platform,
+} = require('internal/process/per_thread');
+
 const {
   markTransferMode,
 } = require('internal/worker/js_transferable');
@@ -85,9 +90,6 @@ const {
 
 const querystring = require('querystring');
 
-const { platform } = process;
-const isWindows = platform === 'win32';
-
 const bindingUrl = internalBinding('url');
 
 const FORWARD_SLASH = /\//g;
@@ -349,7 +351,7 @@ class URLSearchParams {
           throw new ERR_ARG_NOT_ITERABLE('Query pairs');
         }
 
-        // The following implementationd differs from the URL specification:
+        // The following implementation differs from the URL specification:
         // Sequences must first be converted from ECMAScript objects before
         // and operations are done on them, and the operation of converting
         // the sequences would first exhaust the iterators. If the iterator
@@ -367,7 +369,7 @@ class URLSearchParams {
             if (pair.length !== 2) {
               throw new ERR_INVALID_TUPLE('Each query pair', '[name, value]');
             }
-            // Append (innerSequence[0], innerSequence[1]) to querys list.
+            // Append (innerSequence[0], innerSequence[1]) to query's list.
             ArrayPrototypePush(
               this.#searchParams,
               StringPrototypeToWellFormed(`${pair[0]}`),
@@ -1167,6 +1169,12 @@ ObjectDefineProperties(URL, {
     writable: true,
     enumerable: true,
   },
+  parse: {
+    __proto__: null,
+    configurable: true,
+    writable: true,
+    enumerable: true,
+  },
 });
 
 function installObjectURLMethods() {
@@ -1532,7 +1540,12 @@ function pathToFileURL(filepath, options = kEmptyObject) {
   if ((windows ?? isWindows) && StringPrototypeStartsWith(filepath, '\\\\')) {
     const outURL = new URL('file://');
     // UNC path format: \\server\share\resource
-    const hostnameEndIndex = StringPrototypeIndexOf(filepath, '\\', 2);
+    // Handle extended UNC path and standard UNC path
+    // "\\?\UNC\" path prefix should be ignored.
+    // Ref: https://learn.microsoft.com/en-us/windows/win32/fileio/maximum-file-path-limitation
+    const isExtendedUNC = StringPrototypeStartsWith(filepath, '\\\\?\\UNC\\');
+    const prefixLength = isExtendedUNC ? 8 : 2;
+    const hostnameEndIndex = StringPrototypeIndexOf(filepath, '\\', prefixLength);
     if (hostnameEndIndex === -1) {
       throw new ERR_INVALID_ARG_VALUE(
         'path',
@@ -1547,7 +1560,7 @@ function pathToFileURL(filepath, options = kEmptyObject) {
         'Empty UNC servername',
       );
     }
-    const hostname = StringPrototypeSlice(filepath, 2, hostnameEndIndex);
+    const hostname = StringPrototypeSlice(filepath, prefixLength, hostnameEndIndex);
     outURL.hostname = domainToASCII(hostname);
     outURL.pathname = encodePathChars(
       RegExpPrototypeSymbolReplace(backslashRegEx, StringPrototypeSlice(filepath, hostnameEndIndex), '/'),
