@@ -24,7 +24,6 @@ const {
   AbortError,
   codes: {
     ERR_ILLEGAL_CONSTRUCTOR,
-    ERR_INVALID_ARG_TYPE,
     ERR_INVALID_THIS,
   },
 } = require('internal/errors');
@@ -33,6 +32,7 @@ const {
   validateAbortSignal,
   validateBoolean,
   validateObject,
+  validateNumber,
 } = require('internal/validators');
 
 const {
@@ -40,6 +40,7 @@ const {
 } = require('internal/util');
 
 const kScheduler = Symbol('kScheduler');
+let kResistStopPropagation;
 
 function cancelListenerHandler(clear, reject, signal) {
   if (!this._destroyed) {
@@ -49,39 +50,39 @@ function cancelListenerHandler(clear, reject, signal) {
 }
 
 function setTimeout(after, value, options = kEmptyObject) {
-  const args = value !== undefined ? [value] : value;
-  if (options == null || typeof options !== 'object') {
-    return PromiseReject(
-      new ERR_INVALID_ARG_TYPE(
-        'options',
-        'Object',
-        options));
-  }
-  const { signal, ref = true } = options;
   try {
-    validateAbortSignal(signal, 'options.signal');
+    if (typeof after !== 'undefined') {
+      validateNumber(after, 'delay');
+    }
+
+    validateObject(options, 'options');
+
+    if (typeof options?.signal !== 'undefined') {
+      validateAbortSignal(options.signal, 'options.signal');
+    }
+
+    if (typeof options?.ref !== 'undefined') {
+      validateBoolean(options.ref, 'options.ref');
+    }
   } catch (err) {
     return PromiseReject(err);
   }
-  if (typeof ref !== 'boolean') {
-    return PromiseReject(
-      new ERR_INVALID_ARG_TYPE(
-        'options.ref',
-        'boolean',
-        ref));
-  }
+
+  const { signal, ref = true } = options;
 
   if (signal?.aborted) {
     return PromiseReject(new AbortError(undefined, { cause: signal.reason }));
   }
+
   let oncancel;
   const ret = new Promise((resolve, reject) => {
-    const timeout = new Timeout(resolve, after, args, false, ref);
+    const timeout = new Timeout(resolve, after, [value], false, ref);
     insert(timeout, timeout._idleTimeout);
     if (signal) {
       oncancel = FunctionPrototypeBind(cancelListenerHandler,
                                        timeout, clearTimeout, reject, signal);
-      signal.addEventListener('abort', oncancel);
+      kResistStopPropagation ??= require('internal/event_target').kResistStopPropagation;
+      signal.addEventListener('abort', oncancel, { __proto__: null, [kResistStopPropagation]: true });
     }
   });
   return oncancel !== undefined ?
@@ -91,30 +92,26 @@ function setTimeout(after, value, options = kEmptyObject) {
 }
 
 function setImmediate(value, options = kEmptyObject) {
-  if (options == null || typeof options !== 'object') {
-    return PromiseReject(
-      new ERR_INVALID_ARG_TYPE(
-        'options',
-        'Object',
-        options));
-  }
-  const { signal, ref = true } = options;
   try {
-    validateAbortSignal(signal, 'options.signal');
+    validateObject(options, 'options');
+
+    if (typeof options?.signal !== 'undefined') {
+      validateAbortSignal(options.signal, 'options.signal');
+    }
+
+    if (typeof options?.ref !== 'undefined') {
+      validateBoolean(options.ref, 'options.ref');
+    }
   } catch (err) {
     return PromiseReject(err);
   }
-  if (typeof ref !== 'boolean') {
-    return PromiseReject(
-      new ERR_INVALID_ARG_TYPE(
-        'options.ref',
-        'boolean',
-        ref));
-  }
+
+  const { signal, ref = true } = options;
 
   if (signal?.aborted) {
     return PromiseReject(new AbortError(undefined, { cause: signal.reason }));
   }
+
   let oncancel;
   const ret = new Promise((resolve, reject) => {
     const immediate = new Immediate(resolve, [value]);
@@ -123,7 +120,8 @@ function setImmediate(value, options = kEmptyObject) {
       oncancel = FunctionPrototypeBind(cancelListenerHandler,
                                        immediate, clearImmediate, reject,
                                        signal);
-      signal.addEventListener('abort', oncancel);
+      kResistStopPropagation ??= require('internal/event_target').kResistStopPropagation;
+      signal.addEventListener('abort', oncancel, { __proto__: null, [kResistStopPropagation]: true });
     }
   });
   return oncancel !== undefined ?
@@ -133,13 +131,29 @@ function setImmediate(value, options = kEmptyObject) {
 }
 
 async function* setInterval(after, value, options = kEmptyObject) {
-  validateObject(options, 'options');
-  const { signal, ref = true } = options;
-  validateAbortSignal(signal, 'options.signal');
-  validateBoolean(ref, 'options.ref');
+  try {
+    if (typeof after !== 'undefined') {
+      validateNumber(after, 'delay');
+    }
 
-  if (signal?.aborted)
+    validateObject(options, 'options');
+
+    if (typeof options?.signal !== 'undefined') {
+      validateAbortSignal(options.signal, 'options.signal');
+    }
+
+    if (typeof options?.ref !== 'undefined') {
+      validateBoolean(options.ref, 'options.ref');
+    }
+  } catch (err) {
+    return PromiseReject(err);
+  }
+
+  const { signal, ref = true } = options;
+
+  if (signal?.aborted) {
     throw new AbortError(undefined, { cause: signal?.reason });
+  }
 
   let onCancel;
   let interval;
@@ -164,7 +178,8 @@ async function* setInterval(after, value, options = kEmptyObject) {
           callback = undefined;
         }
       };
-      signal.addEventListener('abort', onCancel, { once: true });
+      kResistStopPropagation ??= require('internal/event_target').kResistStopPropagation;
+      signal.addEventListener('abort', onCancel, { __proto__: null, once: true, [kResistStopPropagation]: true });
     }
 
     while (!signal?.aborted) {
@@ -212,7 +227,7 @@ class Scheduler {
   wait(delay, options) {
     if (!this[kScheduler])
       throw new ERR_INVALID_THIS('Scheduler');
-    return setTimeout(delay, undefined, { signal: options?.signal });
+    return setTimeout(delay, undefined, options);
   }
 }
 
